<?php
/*
 * Session Management for PHP3
 *
 * (C) Copyright 1998 SH Online Dienst GmbH
 *                    Boris Erdmann, Kristian Koehntopp
 *
 * $Id: session.inc,v 1.11 1998/11/08 00:37:18 sas Exp $
 *
 */ 

class DB_SAM extends DB_Sql {
  var $Host     = "localhost";        ## DB server is running on localhost
  var $Database = "sam";              ## and we use "sam" as database
  var $User     = "webuser";          ## Connect as this user "webuser"
  var $Password = "";                 ## using no password
}

class Session {
  var $classname = "Session";         ## Needed for object serialization.

  ## Define the parameters of your session by either overwriting
  ## these values or by subclassing session (recommended).

  var $magic = "";                    ## Some string you should change.
  var $mode = "cookie";               ## We propagate session IDs with cookies
  var $fallback_mode;                 ## If this doesn't work, fall back...
  var $lifetime = 0;                  ## 0 = do session cookies, else minutes

  var $gc_time  = 1440;               ## Purge all session data older than 1440 minutes.
  var $gc_probability = 1;            ## Garbage collect probability in percent

  var $database_class = "DB_SAM";     ## Which database to connect...
  var $database_table = "active_sessions"; ## Table to use for session data.

  var $database_lock_semaphore = "";  ## if not an empty string: use this
                                      ## semaphore string to lock db access
  var $auto_init = "";                ## Name of the autoinit-File, if any.
  var $secure_auto_init = 1;          ## Set to 0 only, if all pages call page_close() guaranteed.

  ##
  ## End of parameters.
  ##

  var $name;                          ## Session name
  var $id;                            ## Unique Session ID

  var $db;                            ## Database Object
  var $pt = array();                  ## This Array contains the registered things
  var $in = false;                    ## Marker: Did we already include the autoinit file?

  ## register($things):
  ##
  ## call this function to register the things that should become persistent

  function register($things) {
    $things = explode(",",$things);
    reset($things);
    while ( list(,$thing) = each($things) ) {
      $thing=trim($thing);
      if ( $thing ) {
        $this->pt[$thing] = true;
      }
    }
  }

  function unregister($things) {
    $things = explode(",", $things);
    reset($things);
    while (list(,$thing) = each($things)) {
      $thing = trim($thing);
      if ($thing) {
        unset($this->pt[$thing]);
      }
    }
  }

  ## get_id():
  ##
  ## Propagate the session id according to mode and lifetime.
  ## Will create a new id if necessary. To take over abandoned sessions,
  ## one may provide the new session id as a parameter (not recommended).

  function get_id($id = "") {
    global $HTTP_COOKIE_VARS, $HTTP_GET_VARS, $QUERY_STRING;
    $newid=true;
 
	$this->name = $this->cookiename==""?$this->classname:$this->cookiename;
 
    if ( "" == $id ) {
      $newid=false;
      switch ($this->mode) {
        case "get":
          $id = isset($HTTP_GET_VARS[$this->name]) ? $HTTP_GET_VARS[$this->name] : "";
        break;
        case "cookie":
          $id = isset($HTTP_COOKIE_VARS[$this->name]) ? $HTTP_COOKIE_VARS[$this->name] : "";
        break;
        default:
          die("This has not been coded yet.");
        break;
      }
    }
 
    if ( "" == $id ) {
      $newid=true;
      $id = md5(uniqid($this->magic));
    }
 
    switch ($this->mode) {
      case "cookie":
        if ( $newid && ( 0 == $this->lifetime ) ) {
          SetCookie($this->name, $id, 0, "/");
        }
        if ( 0 < $this->lifetime ) {
          SetCookie($this->name, $id, time()+$this->lifetime*60, "/");
        }
      break;
      case "get":
        if ( isset($QUERY_STRING) ) {
          $QUERY_STRING = ereg_replace(
            "(^|&)".quotemeta(urlencode($this->name))."=".$id."(&|$)",
            "\\1", $QUERY_STRING);
        }
      break;
      default:
        ;
      break;
    }
 
    $this->id = $id;
  }

  ## put_id():
  ## 
  ## Stop using the current session id (unset cookie, ...) and
  ## abandon a session.
  function put_id() {
    global $HTTP_COOKIE_VARS;
 
	$this->name = $this->cookiename==""?$this->classname:$this->cookiename;
    $this->name = $this->classname;
 
    switch ($this->mode) {
      case "inline":
        die("This has not been coded yet.");
      break;
 
      case "get":
        die("This has not been coded yet.");
      break;
 
      default:
            SetCookie($this->name, "", 0, "/");
            $HTTP_COOKIE_VARS[$this->name] = "";
      break;
    }
  }

  ## delete():
  ##
  ## Delete the current session record and put the session id.
  
  function delete() {
    $query = sprintf("delete from %s where name = '%s' and sid = '%s'",
      $this->database_table,
      $this->name,
      $this->id);
    $this->db->query($query);
    $this->put_id();
  }

  ## url($url):
  ##
  ## Helper function: returns $url concatenated with the current 
  ## session $id.
   
  function url($url){
    $url=ereg_replace("[&?]+$", "", $url);
 
    switch ($this->mode) {
      case "get":
 
        $url .= ( ereg("\?", $url) ?  "&" : "?" ).
                urlencode($this->name)."=".$this->id;
 
      break;
      default:
        ;
      break;
    }
    return $url;
  }

  function purl($url) {
    print $this->url($url);
  }

  function self_url() {
    global $PHP_SELF, $QUERY_STRING;

    return $this->url($PHP_SELF.
           ((isset($QUERY_STRING) && ("" != $QUERY_STRING)) ? "?".$QUERY_STRING : ""));
  }

  function pself_url() {
    print $this->self_url();
  }

  ## serialize($prefix,&$str):
  ##
  ## appends a serialized representation of $$prefix
  ## at the end of $str.
  ##
  ## To be able to serialize an object, the object must implement
  ## a variable $classname (containing the name of the class as string)
  ## and a variable $persistent_slots (containing the names of the slots
  ## to be saved as an array of strings).
  ##
  ## You don't need to know...
  function serialize($prefix, $str) {
    static $t,$l,$k;

    ## Determine the type of $$prefix
    eval("\$t = gettype(\$$prefix);");
    switch ( $t ) {

      case "array":
        ## $$prefix is an array. Enumerate the elements and serialize them.
        eval("reset(\$$prefix); \$l = gettype(list(\$k)=each(\$$prefix));");
        $str .= "\$$prefix = array(); ";
        while ( "array" == $l ) {
          ## Structural recursion
          $this->serialize($prefix."[\"".ereg_replace("([\"$])","\\\\1",$k)."\"]", &$str);
          eval("\$l = gettype(list(\$k)=each(\$$prefix));");
        }

      break;
      case "object":
        ## $$prefix is an object. Enumerate the slots and serialize them.
        eval("\$k = \$${prefix}->classname; \$l = reset(\$${prefix}->persistent_slots);");
        $str.="\$$prefix = new $k; ";
        while ( $l ) {
          ## Structural recursion.
          $this->serialize($prefix."->".$l,&$str);
          eval("\$l = next(\$${prefix}->persistent_slots);");
        }

      break;
      default:
        ## $$prefix is an atom. Extract it to $l, then generate code.
        eval("\$l = \$$prefix;");
        $str.="\$$prefix = \"".ereg_replace("([\"$])","\\\\1",$l)."\"; ";


      break;
    }
  }

  function get_lock() {
    if ( "" != $this->database_lock_semaphore ) {
      while ( ! $this->db->query("select get_lock('%s')", $this->database_lock_semaphore) ) {
        $t = 1 + time(); while ( $t > time() ) { ; }
      }
    }
  }

  function release_lock() {
    if ( "" != $this->database_lock_semaphore ) {
      $this->db->query("select release_lock('%s')", $this->database_lock_semaphore);
    }
  }

  ## freeze():
  ##
  ## freezes all registered things ( scalar variables, arrays, objects ) into
  ## a database table

  function freeze() {
    $str="";

    $this->serialize("this->in",&$str);
    $this->serialize("this->pt",&$str);

    reset($this->pt);
    while ( list($thing) = each($this->pt) ) {
      $thing=trim($thing);
      if ( $thing ) {
        $this->serialize("GLOBALS[\"".$thing."\"]",&$str);
      }
    }

    $now = date("YmdHis");
    $query = sprintf("update %s set val='%s',changed='%s' where sid='%s' and name='%s'",
                             $this->database_table,
                             addslashes($str),
                             $now,
                             $this->id,
                             $this->classname);
    $this->db->query($query);
    if ( ! $this->db->affected_rows() ) {
      $query = sprintf("insert into %s ( sid, name, val, changed )
                             values ( '%s', '%s', '%s', '%s')",

                             $this->database_table,
                             $this->id,
                             $this->classname,
                             addslashes($str),
                             $now);
      if ( ! $this->db->query($query) ) {
        $this->release_lock();
        $this->db->halt("Session: Can't freeze().");
      }
    }
    $this->release_lock(); 
  }

  ## thaw:
  ##
  ## Reload frozen variables from the database and microwave them.

  function thaw() {

    $this->get_lock();

    $this->db->query(sprintf("select val
                                from %s
                               where sid  = '%s'
                                 and name = '%s'",
 
                     $this->database_table,
                     $this->id,
                     $this->classname));
 
    while ( $this->db->next_record() ) {
#     printf("Eval: %s<br>\n", $this->db->f("val"));
      eval("; ".stripslashes($this->db->f("val")));
    }
  }

  ##
  ## Garbage collection
  ##
  ## Destroy all session data older than this
  ##
  function gc() {
    $timeout = mktime(date("H"), date("i"), date("s"),
                      date("m"), date("d"), date("Y"));
    $sqldate = date("YmdHis", $timeout-($this->gc_time*60));
    $query = sprintf("delete from %s where changed < '%s' and name = '%s'",
      $this->database_table, $sqldate, $this->name);
    $this->db->query($query);
  }

  ##
  ## Variable precedence functions
  ##
  function reimport_get_vars() {
    $this->reimport_any_vars("HTTP_GET_VARS");
  }

  function reimport_post_vars() {
    $this->reimport_any_vars("HTTP_POST_VARS");
  }

  function reimport_cookie_vars() {
    $this->reimport_any_vars("HTTP_COOKIE_VARS");
  }

  function reimport_any_vars($arrayname) {
    global $$arrayname;

    if (!is_array($$arrayname))
      return;

    reset($$arrayname);
    while(list($key, $val) = each($$arrayname)) {
      $GLOBALS[$key] = $val;
    }
  }

  ##
  ## Initialization
  ##

  function start($sid = "") {
    global $HTTP_COOKIE_VARS, $HTTP_GET_VARS, $HTTP_HOST, $HTTPS;
    
	$this->name = $this->cookiename==""?$this->classname:$this->cookiename;
	
	if (   isset($this->fallback_mode)
      && ( "get" == $this->fallback_mode ) 
      && ( "cookie" == $this->mode )
      && ( ! isset($HTTP_COOKIE_VARS[$this->name]) ) ) {
 
      if ( isset($HTTP_GET_VARS[$this->name]) ) {
        $this->mode = $this->fallback_mode;
      } else {
        header("Status: 302 Moved Temporarily");
        $this->get_id($sid);
        $this->mode = $this->fallback_mode;
        if( isset($HTTPS) && $HTTPS == 'on' ){
          ## You will need to fix suexec as well, if you use Apache and CGI PHP
          $PROTOCOL='https';
        } else {
          $PROTOCOL='http';
        }
          header("Location: ". $PROTOCOL. "://". $HTTP_HOST.$this->self_url());
        exit;
      }
    }

    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
    header("Cache-Control: no-cache, must-revalidate");
	if(!isset($this->allowcache)) {
		header("Pragma: no-cache");
		header("Expires: 0");
	}
 
    $this->get_id($sid);
 
    eval("\$this->db = new ".$this->database_class.";");


    $this->thaw();

    ## Garbage collect, if necessary
    srand(time());
    if ((rand()%100) < $this->gc_probability) {
      $this->gc();
    }
  }
}
?>
